@@ -64,14 +64,15 @@ def folder_worker(A):
             break
         else:
             dicom_adder = AddDicomToDictionary()
-            dicom_path, images_dictionary, rt_dictionary, rd_dictionary, verbose = item
+            dicom_path, images_dictionary, rt_dictionary, rd_dictionary, rp_dictionary, verbose = item
             try:
                 if verbose:
                     print('Loading from {}'.format(dicom_path))
                 dicom_adder.add_dicom_to_dictionary_from_path(dicom_path=dicom_path,
                                                               images_dictionary=images_dictionary,
                                                               rt_dictionary=rt_dictionary,
-                                                              rd_dictionary=rd_dictionary)
+                                                              rd_dictionary=rd_dictionary,
+                                                              rp_dictionary=rp_dictionary)
             except:
                 print('failed on {}'.format(dicom_path))
             pbar.update()
@@ -161,6 +162,15 @@ def add_images_to_dictionary(images_dictionary, sitk_dicom_reader, path: typing.
         images_dictionary[series_instance_uid] = temp_dict
 
 
+def add_rp_to_dictionary(ds, path: typing.Union[str, bytes, os.PathLike], rp_dictionary):
+    series_instance_uid = ds.SeriesInstanceUID
+    if series_instance_uid not in rp_dictionary:
+        for referenced_structureset in ds.ReferencedStructureSetSequence:
+            refed_structure_uid = referenced_structureset.ReferencedSOPInstanceUID
+            temp_dict = {'Path': path, 'ReferencedStructureUID': refed_structure_uid}
+            rp_dictionary[series_instance_uid] = temp_dict
+
+
 def add_rt_to_dictionary(ds, path: typing.Union[str, bytes, os.PathLike], rt_dictionary):
     """
     :param ds: pydicom data structure
@@ -183,7 +193,7 @@ def add_rt_to_dictionary(ds, path: typing.Union[str, bytes, os.PathLike], rt_dic
                         if Structures.ROIName not in rois_in_structure:
                             rois_in_structure[Structures.ROIName] = Structures.ROINumber
                     temp_dict = {'Path': path, 'ROI_Names': rois, 'ROIs_in_structure': rois_in_structure,
-                                 'SeriesInstanceUID': refed_series_instance_uid}
+                                 'SeriesInstanceUID': refed_series_instance_uid, 'Plans': []}
                     rt_dictionary[series_instance_uid] = temp_dict
 
 
@@ -222,7 +232,7 @@ def add_sops_to_dictionary(sitk_dicom_reader, series_instances_dictionary):
 
 
 def return_template_dictionary():
-    template_dictionary = {'Image_Path': None, 'PatientID': None, 'RTs': {}, 'RDs': {}, 'Description': None,
+    template_dictionary = {'Image_Path': None, 'PatientID': None, 'RTs': {}, 'RDs': {}, 'RPs': {}, 'Description': None,
                            'SOP_Instance_UIDs': None, 'SeriesInstanceUID': None}
     return template_dictionary
 
@@ -246,13 +256,13 @@ class AddDicomToDictionary(object):
         self.reader = sitk.ImageSeriesReader()
         self.reader.GlobalWarningDisplayOff()
 
-    def add_dicom_to_dictionary_from_path(self, dicom_path, images_dictionary, rt_dictionary, rd_dictionary):
+    def add_dicom_to_dictionary_from_path(self, dicom_path, images_dictionary, rt_dictionary, rd_dictionary, rp_dictionary):
         fileList = glob(os.path.join(dicom_path, '*.dcm'))
         series_ids = self.reader.GetGDCMSeriesIDs(dicom_path)
         all_names = []
         for series_id in series_ids:
             dicom_names = self.reader.GetGDCMSeriesFileNames(dicom_path, series_id)
-            all_names += [os.path.split(i)[-1] for i in dicom_names]
+            all_names += dicom_names
             self.image_reader.SetFileName(dicom_names[0])
             self.image_reader.Execute()
             modality = self.image_reader.GetMetaData("0008|0060")
@@ -268,6 +278,8 @@ class AddDicomToDictionary(object):
             modality = rt.Modality
             if modality.lower().find('struct') != -1:
                 add_rt_to_dictionary(ds=rt, path=lstRSFile, rt_dictionary=rt_dictionary)
+            elif modality.lower().find('plan') != -1:
+                add_rp_to_dictionary(ds=rt, path=lstRSFile, rp_dictionary=rp_dictionary)
 
 
 class DicomReaderWriter(object):
@@ -293,6 +305,7 @@ class DicomReaderWriter(object):
         self.rt_dictionary = {}
         self.images_dictionary = {}
         self.rd_dictionary = {}
+        self.rp_dictionary = {}
         if Contour_Names is None:
             Contour_Names = []
         if associations is None:
@@ -488,6 +501,30 @@ class DicomReaderWriter(object):
                 print(roi)
         return self.all_rois
 
+    def return_files_from_UID(self, UID: str) -> List[str]:
+        """
+        Args:
+            UID: A string UID found in images_dictionary.
+
+        Returns:
+            file_list: A list of file paths that are associated with that UID, being images, RTs, RDs, and RPs
+        """
+        out_file_paths = list()
+        if UID not in self.images_dictionary:
+            print(UID + " Not found in dictionary")
+            return out_file_paths
+        image_dictionary = self.images_dictionary[UID]
+        dicom_path = image_dictionary['Image_Path']
+        image_reader = sitk.ImageFileReader()
+        image_reader.LoadPrivateTagsOn()
+        reader = sitk.ImageSeriesReader()
+        reader.GlobalWarningDisplayOff()
+        out_file_paths += reader.GetGDCMSeriesFileNames(dicom_path, UID)
+        for key in ['RTs', 'RDs']:
+            for structure_key in image_dictionary[key]:
+                out_file_paths += [image_dictionary[key][structure_key]['Path']]
+        return out_file_paths
+
     def where_are_RTs(self, ROIName: str) -> None:
         print('Please move over to using .where_is_ROI(), as this better represents the definition')
         self.where_is_ROI(ROIName=ROIName)
@@ -553,7 +590,8 @@ class DicomReaderWriter(object):
                 t.start()
                 threads.append(t)
             for index, path in enumerate(paths_with_dicom):
-                item = [path, self.images_dictionary, self.rt_dictionary, self.rd_dictionary, self.verbose]
+                item = [path, self.images_dictionary, self.rt_dictionary, self.rd_dictionary, self.rp_dictionary,
+                        self.verbose]
                 q.put(item)
             for i in range(thread_count):
                 q.put(None)
